@@ -1,7 +1,7 @@
 __author__ = 'Altertech, http://www.altertech.com/'
 __copyright__ = 'Copyright (C) 2018-2020 Altertech Group'
 __license__ = 'Apache License 2.0'
-__version__ = '2.0.23'
+__version__ = '2.0.25'
 
 import platform
 import os
@@ -441,6 +441,10 @@ def is_use_limits():
 
 
 def convert_level(level):
+    try:
+        level = int(level)
+    except:
+        pass
     if level in [10, 20, 30, 40, 50]: return level
     elif isinstance(level, str):
         level = level.lower()
